@@ -22,7 +22,11 @@
  * Boston, MA 02111-1307, USA.  
  */
 
-#include <libgnome/gnome-macros.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include "gdl-i18n.h"
 #include <gdk/gdkx.h>
 
 #include "gdl-tools.h"
@@ -47,8 +51,7 @@ static void     gdl_dock_master_get_property  (GObject            *object,
                                                GValue             *value,
                                                GParamSpec         *pspec);
 
-static gboolean _gdl_dock_master_remove       (gpointer            key,
-                                               GdlDockObject      *object,
+static void     _gdl_dock_master_remove       (GdlDockObject      *object,
                                                GdlDockMaster      *master);
 
 static void     gdl_dock_master_drag_begin    (GdlDockItem        *item, 
@@ -91,14 +94,29 @@ struct _GdlDockMasterPrivate {
     GdlDock        *rect_owner;
     
     GdlDockRequest *drag_request;
+
+    /* source id for the idle handler to emit a layout_changed signal */
+    guint           idle_layout_changed_id;
+
+    /* hashes to quickly calculate the overall locked status: i.e.
+     * if size(unlocked_items) == 0 then locked = 1
+     * else if size(locked_items) == 0 then locked = 0
+     * else locked = -1
+     */
+    GHashTable     *locked_items;
+    GHashTable     *unlocked_items;
 };
 
+#define COMPUTE_LOCKED(master)                                          \
+    (g_hash_table_size ((master)->_priv->unlocked_items) == 0 ? 1 :     \
+     (g_hash_table_size ((master)->_priv->locked_items) == 0 ? 0 : -1))
+
 static guint master_signals [LAST_SIGNAL] = { 0 };
 
 
 /* ----- Private interface ----- */
 
-GNOME_CLASS_BOILERPLATE (GdlDockMaster, gdl_dock_master, GObject, G_TYPE_OBJECT);
+GDL_CLASS_BOILERPLATE (GdlDockMaster, gdl_dock_master, GObject, G_TYPE_OBJECT);
 
 static void
 gdl_dock_master_class_init (GdlDockMasterClass *klass)
@@ -152,19 +170,16 @@ gdl_dock_master_instance_init (GdlDockMaster *master)
     
     master->_priv = g_new0 (GdlDockMasterPrivate, 1);
     master->_priv->number = 1;
+
+    master->_priv->locked_items = g_hash_table_new (g_direct_hash, g_direct_equal);
+    master->_priv->unlocked_items = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
-static gboolean
-_gdl_dock_master_remove (gpointer       key,
-                         GdlDockObject *object,
+static void
+_gdl_dock_master_remove (GdlDockObject *object,
                          GdlDockMaster *master)
 {
-    gboolean retval = (key != NULL);
-    
-    /* do all the stuff we should for removing the object from our
-       object, but don't remove it from the hash, so we can use this
-       function for finalizing the object */
-    g_return_val_if_fail (master != NULL && object != NULL, FALSE);
+    g_return_if_fail (master != NULL && object != NULL);
 
     if (GDL_IS_DOCK (object)) {
         GList *found_link;
@@ -199,31 +214,27 @@ _gdl_dock_master_remove (gpointer       key,
             }
         }
     }
-    else if (GDL_IS_DOCK_ITEM (object)) {
-        /* disconnect dock dragging signals */
-        g_signal_handlers_disconnect_matched (object, G_SIGNAL_MATCH_DATA, 
-                                              0, 0, NULL, NULL, master);
-    }
+    /* disconnect dock object signals */
+    g_signal_handlers_disconnect_matched (object, G_SIGNAL_MATCH_DATA, 
+                                          0, 0, NULL, NULL, master);
 
-    object->master = NULL;
-    if (key) {
-        /* we were called from the hash iterator, so the object is in the hash */
-        g_object_unref (object);
-    }
-    else {
-        /* unref the object from the hash if it's there */
-        if (object->name) {
-            GdlDockObject *found_object;
-            found_object = g_hash_table_lookup (master->dock_objects, object->name);
-            if (found_object == object) {
-                g_object_unref (object);
-                /* will remove the object later */
-                retval = TRUE;
-            }
+    /* unref the object from the hash if it's there */
+    if (object->name) {
+        GdlDockObject *found_object;
+        found_object = g_hash_table_lookup (master->dock_objects, object->name);
+        if (found_object == object) {
+            g_hash_table_remove (master->dock_objects, object->name);
+            g_object_unref (object);
         }
     }
+}
 
-    return retval;
+static void
+ht_foreach_build_slist (gpointer  key,
+                        gpointer  value,
+                        GSList  **slist)
+{
+    *slist = g_slist_prepend (*slist, value);
 }
 
 static void
@@ -242,7 +253,23 @@ gdl_dock_master_dispose (GObject *g_object)
         master->toplevel_docks = NULL;
     }
     
+    if (master->dock_objects) {
+        GSList *alive_docks = NULL;
+        g_hash_table_foreach (master->dock_objects,
+                              (GHFunc) ht_foreach_build_slist, &alive_docks);
+        while (alive_docks) {
+            gdl_dock_object_unbind (GDL_DOCK_OBJECT (alive_docks->data));
+            alive_docks = g_slist_delete_link (alive_docks, alive_docks);
+        }
+        
+        g_hash_table_destroy (master->dock_objects);
+        master->dock_objects = NULL;
+    }
+    
     if (master->_priv) {
+        if (master->_priv->idle_layout_changed_id)
+            g_source_remove (master->_priv->idle_layout_changed_id);
+        
         if (master->_priv->root_xor_gc) {
             g_object_unref (master->_priv->root_xor_gc);
             master->_priv->root_xor_gc = NULL;
@@ -255,19 +282,17 @@ gdl_dock_master_dispose (GObject *g_object)
         }
         g_free (master->_priv->default_title);
         master->_priv->default_title = NULL;
+
+        g_hash_table_destroy (master->_priv->locked_items);
+        master->_priv->locked_items = NULL;
+        g_hash_table_destroy (master->_priv->unlocked_items);
+        master->_priv->unlocked_items = NULL;
         
         g_free (master->_priv);
         master->_priv = NULL;
     }
 
-    if (master->dock_objects) {
-        g_hash_table_foreach_remove (master->dock_objects,
-                                     (GHRFunc) _gdl_dock_master_remove, master);
-        g_hash_table_destroy (master->dock_objects);
-        master->dock_objects = NULL;
-    }
-    
-    GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (g_object));
+    GDL_CALL_PARENT (G_OBJECT_CLASS, dispose, (g_object));
 }
 
 static void 
@@ -325,49 +350,6 @@ gdl_dock_master_set_property  (GObject      *object,
     }
 }
 
-static gint
-is_locked (GdlDockItem *item)
-{
-    gint     result;
-    gboolean item_locked;
-    g_object_get (item, "locked", &item_locked, NULL);
-    result = item_locked ? 1 : 0;
-
-    if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (item))) {
-        GList *children, *l;
-        children = gtk_container_get_children (GTK_CONTAINER (item));
-        /* compare with our children */
-        for (l = children; l; l = l->next) {
-            if (is_locked (GDL_DOCK_ITEM (l->data)) != result) {
-                result = -1;
-                break;
-            }
-        }
-        g_list_free (children);
-    }
-    return result;
-}
-
-static gint
-gdl_dock_master_locked (GdlDockMaster *master)
-{
-    gint   result = -1;
-    GList *l;
-    
-    /* see if all items are locked or unlocked */
-    for (l = master->toplevel_docks; l; l = l->next) {
-        GdlDock *dock = GDL_DOCK (l->data);
-        if (dock->root) {
-            gint r = is_locked (GDL_DOCK_ITEM (dock->root));
-            if (r == -1 || (r != result && result != -1))
-                return -1;
-            /* save the result for comparision */
-            result = r;
-        }
-    }
-    return result;
-}
-
 static void
 gdl_dock_master_get_property  (GObject      *object,
                                guint         prop_id,
@@ -381,7 +363,7 @@ gdl_dock_master_get_property  (GObject      *object,
             g_value_set_string (value, master->_priv->default_title);
             break;
         case PROP_LOCKED:
-            g_value_set_int (value, gdl_dock_master_locked (master));
+            g_value_set_int (value, COMPUTE_LOCKED (master));
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -479,9 +461,10 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
         GtkWidget *widget;
         /* ok, now get the widget who owns that window and see if we can
            get to a GdlDock by walking up the hierarchy */
-        gdk_window_get_user_data (window, (gpointer *) &widget);
+        gdk_window_get_user_data (window, (gpointer) &widget);
         if (GTK_IS_WIDGET (widget)) {
-            while (widget && !GDL_IS_DOCK (widget))
+            while (widget && (!GDL_IS_DOCK (widget) || 
+	           GDL_DOCK_OBJECT_GET_MASTER (widget) != master))
                 widget = widget->parent;
             if (widget) {
                 gint win_w, win_h;
@@ -526,7 +509,6 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
     }
     
     if (!may_dock) {
-        GdkRectangle *rect;
         GtkRequisition req;
         
         dock = NULL;
@@ -546,9 +528,7 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
             g_value_unset (&my_request.extra);
 
         g_value_init (&my_request.extra, GDK_TYPE_RECTANGLE);
-        rect = g_new0 (GdkRectangle, 1);
-        *rect = my_request.rect;
-        g_value_set_boxed (&my_request.extra, rect);
+        g_value_set_boxed (&my_request.extra, &my_request.rect);
     };
 
     if (!(my_request.rect.x == request->rect.x &&
@@ -639,11 +619,94 @@ gdl_dock_master_layout_changed (GdlDockMaster *master)
 {
     g_return_if_fail (GDL_IS_DOCK_MASTER (master));
 
-    /* notify the controller */
+    /* emit "layout_changed" on the controller to notify the user who
+     * normally shouldn't have access to us */
     if (master->controller)
         g_signal_emit_by_name (master->controller, "layout_changed");
+
+    /* remove the idle handler if there is one */
+    if (master->_priv->idle_layout_changed_id) {
+        g_source_remove (master->_priv->idle_layout_changed_id);
+        master->_priv->idle_layout_changed_id = 0;
+    }
 }
 
+static gboolean
+idle_emit_layout_changed (gpointer user_data)
+{
+    GdlDockMaster *master = user_data;
+
+    g_return_val_if_fail (master && GDL_IS_DOCK_MASTER (master), FALSE);
+
+    master->_priv->idle_layout_changed_id = 0;
+    g_signal_emit (master, master_signals [LAYOUT_CHANGED], 0);
+    
+    return FALSE;
+}
+
+static void 
+item_dock_cb (GdlDockObject    *object,
+              GdlDockObject    *requestor,
+              GdlDockPlacement  position,
+              GValue           *other_data,
+              gpointer          user_data)
+{
+    GdlDockMaster *master = user_data;
+    
+    g_return_if_fail (requestor && GDL_IS_DOCK_OBJECT (requestor));
+    g_return_if_fail (master && GDL_IS_DOCK_MASTER (master));
+
+    /* here we are in fact interested in the requestor, since it's
+     * assumed that object will not change its visibility... for the
+     * requestor, however, could mean that it's being shown */
+    if (!GDL_DOCK_OBJECT_IN_REFLOW (requestor) &&
+        !GDL_DOCK_OBJECT_AUTOMATIC (requestor)) {
+        if (!master->_priv->idle_layout_changed_id)
+            master->_priv->idle_layout_changed_id =
+                g_idle_add (idle_emit_layout_changed, master);
+    }
+}
+
+static void 
+item_detach_cb (GdlDockObject *object,
+                gboolean       recursive,
+                gpointer       user_data)
+{
+    GdlDockMaster *master = user_data;
+    
+    g_return_if_fail (object && GDL_IS_DOCK_OBJECT (object));
+    g_return_if_fail (master && GDL_IS_DOCK_MASTER (master));
+
+    if (!GDL_DOCK_OBJECT_IN_REFLOW (object) &&
+        !GDL_DOCK_OBJECT_AUTOMATIC (object)) {
+        if (!master->_priv->idle_layout_changed_id)
+            master->_priv->idle_layout_changed_id =
+                g_idle_add (idle_emit_layout_changed, master);
+    }
+}
+
+static void
+item_notify_cb (GdlDockObject *object,
+                GParamSpec    *pspec,
+                gpointer       user_data)
+{
+    GdlDockMaster *master = user_data;
+    gint locked = COMPUTE_LOCKED (master);
+    gboolean item_locked;
+    
+    g_object_get (object, "locked", &item_locked, NULL);
+
+    if (item_locked) {
+        g_hash_table_remove (master->_priv->unlocked_items, object);
+        g_hash_table_insert (master->_priv->locked_items, object, NULL);
+    } else {
+        g_hash_table_remove (master->_priv->locked_items, object);
+        g_hash_table_insert (master->_priv->unlocked_items, object, NULL);
+    }
+    
+    if (COMPUTE_LOCKED (master) != locked)
+        g_object_notify (G_OBJECT (master), "locked");
+}
 
 /* ----- Public interface ----- */
 
@@ -689,6 +752,12 @@ gdl_dock_master_add (GdlDockMaster *master,
             master->toplevel_docks = g_list_prepend (master->toplevel_docks, object);
         else
             master->toplevel_docks = g_list_append (master->toplevel_docks, object);
+
+        /* we are interested in the dock request this toplevel
+         * receives to update the layout */
+        g_signal_connect (object, "dock",
+                          G_CALLBACK (item_dock_cb), master);
+
     }
     else if (GDL_IS_DOCK_ITEM (object)) {
         /* we need to connect the item's signals */
@@ -698,6 +767,26 @@ gdl_dock_master_add (GdlDockMaster *master,
                           G_CALLBACK (gdl_dock_master_drag_motion), master);
         g_signal_connect (object, "dock_drag_end",
                           G_CALLBACK (gdl_dock_master_drag_end), master);
+        g_signal_connect (object, "dock",
+                          G_CALLBACK (item_dock_cb), master);
+        g_signal_connect (object, "detach",
+                          G_CALLBACK (item_detach_cb), master);
+
+        /* register to "locked" notification if the item has a grip,
+         * and add the item to the corresponding hash */
+        if (GDL_DOCK_ITEM_HAS_GRIP (GDL_DOCK_ITEM (object))) {
+            g_signal_connect (object, "notify::locked",
+                              G_CALLBACK (item_notify_cb), master);
+            item_notify_cb (object, NULL, master);
+        }
+        
+        /* post a layout_changed emission if the item is not automatic
+         * (since it should be added to the items model) */
+        if (!GDL_DOCK_OBJECT_AUTOMATIC (object)) {
+            if (!master->_priv->idle_layout_changed_id)
+                master->_priv->idle_layout_changed_id =
+                    g_idle_add (idle_emit_layout_changed, master);
+        }
     }
 }
 
@@ -705,19 +794,33 @@ void
 gdl_dock_master_remove (GdlDockMaster *master,
                         GdlDockObject *object)
 {
-    gchar *name;
-    
     g_return_if_fail (master != NULL && object != NULL);
 
+    /* remove from locked/unlocked hashes and property change if
+     * that's the case */
+    if (GDL_IS_DOCK_ITEM (object) && GDL_DOCK_ITEM_HAS_GRIP (GDL_DOCK_ITEM (object))) {
+        gint locked = COMPUTE_LOCKED (master);
+        if (g_hash_table_remove (master->_priv->locked_items, object) ||
+            g_hash_table_remove (master->_priv->unlocked_items, object)) {
+            if (COMPUTE_LOCKED (master) != locked)
+                g_object_notify (G_OBJECT (master), "locked");
+        }
+    }
+        
     /* ref the master, since removing the controller could cause master disposal */
     g_object_ref (master);
     
-    name = g_strdup (object->name);
     /* all the interesting stuff happens in _gdl_dock_master_remove */
-    if (_gdl_dock_master_remove (NULL, object, master))
-        g_hash_table_remove (master->dock_objects, name);
-    g_free (name);
+    _gdl_dock_master_remove (object, master);
 
+    /* post a layout_changed emission if the item is not automatic
+     * (since it should be removed from the items model) */
+    if (!GDL_DOCK_OBJECT_AUTOMATIC (object)) {
+        if (!master->_priv->idle_layout_changed_id)
+            master->_priv->idle_layout_changed_id =
+                g_idle_add (idle_emit_layout_changed, master);
+    }
+    
     /* balance ref count */
     g_object_unref (master);
 }
